import { useEditor } from "@/modules/editor";
import { Background, Layout } from "../../../typings";
import { DesignComp } from "@/modules/editor/defines/DesignTemp/DesignComp";
import { reactive } from "vue";

type IOptions<T, C> = {
  name: string;
  compKey?: string;
  thumbnail: string;
  value: T;
  layout?: Layout;
  background?: Background;
  children?: C;
};

export function createOptions<T, C>(options: IOptions<T, C>) {
  function useCompData(compId: string): {
    value: T;
    children: Record<keyof C, DesignComp>;
  } {
    const editor = useEditor();
    return editor.store.designCompMap.get(compId) as any;
  }
  return { options, useCompData };
}

export function createOptions2<T, C>(defaults: IOptions<T, C>) {
  function useCompData(compId: string): {
    id:  string,
    value: T;
    children: Record<keyof C, DesignComp>;
  } {
    const editor = useEditor();
    let data = editor.store.designCompMap.get(compId);
    if (!data || !compId ) {//为空,则创建一个
        data = reactive(new DesignComp(defaults as any));
        editor.store.setCompData(data.id, data);
    }
    return data as any;
  }
  return { options: defaults, useCompData };
}